# Copyright 2023-2024 by Advanced Technology Systems, LLC.
#
# Unless required by applicable law or agreed to in writing, this software is provided on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND.
#
# Licensed by Advanced Technology Systems, LLC.  The full license is available at:
# http://advtechnologysystems.com/license

import boto3
import json

from argparse import ArgumentParser


class APIGatewayDeployer(object):

    def __init__(self, api_id, api_dictionary):
        self._api_id = api_id
        self._api_dictionary = api_dictionary
        self._client = boto3.client("apigateway")

    def close(self):

        self._client.close()

    def deploy_and_finalize(self):

        _ = self.deploy_only()

        if "FullPlanDict" in self._api_dictionary.keys():
            full_plan_dict = self._api_dictionary["FullPlanDict"]
            usage_plan_id = self._client.create_usage_plan(name=full_plan_dict["PlanName"],
                                                           description=full_plan_dict["Description"],
                                                           apiStages=full_plan_dict["StagesList"],
                                                           throttle=full_plan_dict["APIThrottleDict"],
                                                           quota=full_plan_dict["APIQuotaDict"],
                                                           tags=full_plan_dict["APITagsDict"])["id"]
        else:
            api_stages_list = [{"apiId": self._api_id, "stage": self._api_dictionary["DefaultStageName"]}]
            if "Tags" in self._api_dictionary.keys():
                usage_plan_id = self._client.create_usage_plan(name=self._api_dictionary["UsagePlanName"],
                                                               apiStages=api_stages_list,
                                                               tags=self._api_dictionary["Tags"])["id"]
            else:
                usage_plan_id = self._client.create_usage_plan(name=self._api_dictionary["UsagePlanName"],
                                                               apiStages=api_stages_list)["id"]
        if "Tags" in self._api_dictionary.keys():
            api_key_id = self._client.create_api_key(name=self._api_dictionary["APIKeyName"], enabled=True,
                                                     tags=self._api_dictionary["Tags"])["id"]
        else:
            api_key_id = self._client.create_api_key(name=self._api_dictionary["APIKeyName"], enabled=True)["id"]

        r = self._client.create_usage_plan_key(usagePlanId=usage_plan_id, keyId=api_key_id, keyType="API_KEY")

        return r

    def deploy_only(self):

        if "CacheSizeGB" in self._api_dictionary.keys():
            r = self._client.create_deployment(restApiId=self._api_id,
                                               stageName=self._api_dictionary["DefaultStageName"],
                                               cacheClusterEnabled=True,
                                               cacheClusterSize=self._api_dictionary["CacheSizeGB"])
        else:
            r = self._client.create_deployment(restApiId=self._api_id,
                                               stageName=self._api_dictionary["DefaultStageName"])

        return r


if __name__ == "__main__":

    api_client = None

    try:
        parser = ArgumentParser()

        parser.add_argument("--api-id", action="store", dest="api_id", required=True,
                            help="the id of the API")
        parser.add_argument("--file", action="store", dest="file_name", required=True,
                            help="the name of the JSON document containing basic API configuration")

        args = parser.parse_args()

        with open(args.file_name) as json_file:
            api_dict = json.loads(json_file.read())

        api_client = APIGatewayDeployer(args.api_id, api_dict)
        response = api_client.deploy_and_finalize()

        print("API and usage plan deployment complete. "
              "API key named \"{}\" with id {} created.".format(response["name"], response["id"]))

        exit(0)

    except Exception as e:
        print(e)
        exit(-1)
    finally:
        if api_client is not None:
            api_client.close()
