# Copyright 2023-2024 by Advanced Technology Systems, LLC.
#
# Unless required by applicable law or agreed to in writing, this software is provided on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND.
#
# Licensed by Advanced Technology Systems, LLC.  The full license is available at:
# http://advtechnologysystems.com/license

import boto3

from ats.utils.api.aws_api_utils_parser import AWSAPIUtilsParser


class APIGatewayMethodCreator(object):

    def __init__(self, api_id, parent_id, method_dictionary):
        self._api_id = api_id
        self._parent_id = parent_id
        self._method_dictionary = method_dictionary
        self._client = boto3.client("apigateway")

    def close(self):

        self._client.close()

    def create_noauth_api_method(self):

        if "ModelName" in self._method_dictionary.keys():
            request_model_dict = {"application/json": self._method_dictionary["ModelName"]}
        else:
            request_model_dict = {"application/json": None}
        response_model_dict = {"application/json": "Empty"}
        response_template_dict = {"application/json": ""}

        resource_id = self._client.create_resource(restApiId=self._api_id, parentId=self._parent_id,
                                                   pathPart=self._method_dictionary["MethodName"])["id"]

        if "ModelName" in self._method_dictionary.keys():
            _ = self._client.put_method(restApiId=self._api_id, resourceId=resource_id,
                                        httpMethod=self._method_dictionary["HTTPType"],
                                        authorizationType="NONE", apiKeyRequired=True,
                                        requestModels=request_model_dict)
        else:
            _ = self._client.put_method(restApiId=self._api_id, resourceId=resource_id,
                                        httpMethod=self._method_dictionary["HTTPType"],
                                        authorizationType="NONE", apiKeyRequired=True)

        _ = self._client.put_method_response(restApiId=self._api_id, resourceId=resource_id,
                                             httpMethod=self._method_dictionary["HTTPType"],
                                             statusCode="200", responseModels=response_model_dict)

        if self._method_dictionary["HTTPType"].startswith("GET"):
            int_type = "AWS"
        else:
            int_type = "AWS_PROXY"

        if "RequestTemplate" in self._method_dictionary.keys():
            _ = self._client.put_integration(restApiId=self._api_id, resourceId=resource_id,
                                             httpMethod=self._method_dictionary["HTTPType"],
                                             type=int_type,
                                             integrationHttpMethod=self._method_dictionary["IntegrationType"],
                                             uri=self._method_dictionary["URI"],
                                             requestTemplates=self._method_dictionary["RequestTemplate"],
                                             passthroughBehavior="WHEN_NO_MATCH", contentHandling="CONVERT_TO_TEXT")
        else:
            _ = self._client.put_integration(restApiId=self._api_id, resourceId=resource_id,
                                             httpMethod=self._method_dictionary["HTTPType"],
                                             type=int_type,
                                             integrationHttpMethod=self._method_dictionary["IntegrationType"],
                                             uri=self._method_dictionary["URI"], passthroughBehavior="WHEN_NO_MATCH",
                                             contentHandling="CONVERT_TO_TEXT")

        _ = self._client.put_integration_response(restApiId=self._api_id, resourceId=resource_id,
                                                  httpMethod=self._method_dictionary["HTTPType"],
                                                  statusCode="200", responseTemplates=response_template_dict)


if __name__ == "__main__":

    api_client = None

    try:
        parser = AWSAPIUtilsParser()

        parser.add_argument("--api-id", action="store", dest="api_id", required=True,
                            help="the id of the API")
        parser.add_argument("--parent-id", action="store", dest="parent_id", required=True,
                            help="the id of the parent resource (may be root)")

        args = parser.parse_args()

        method_dict = parser.load_config_dict()
        api_client = APIGatewayMethodCreator(args.api_id, args.parent_id, method_dict)
        api_client.create_noauth_api_method()

        print("API method created.")

        exit(0)

    except Exception as e:
        print(e)
        exit(-1)
    finally:
        if api_client is not None:
            api_client.close()
