# Copyright 2023-2024 by Advanced Technology Systems, LLC.
#
# Unless required by applicable law or agreed to in writing, this software is provided on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND.
#
# Licensed by Advanced Technology Systems, LLC.  The full license is available at:
# http://advtechnologysystems.com/license

import boto3
import json

from ats.utils.api.aws_api_utils_parser import AWSAPIUtilsParser


class APIRoleCreator(object):

    def __init__(self, role_dictionary):
        self._role_dictionary = role_dictionary
        self._client = boto3.client("iam")

    def attach_role_policy(self, policy_arn):

        _ = self._client.attach_role_policy(RoleName=self._role_dictionary["Name"], PolicyArn=policy_arn)

    def close(self):
        self._client.close()

    def create_role(self):

        assume_role_doc = json.dumps(self._role_dictionary["AssumeRoleDoc"]).strip()

        if "Description" in self._role_dictionary.keys():
            r = self._client.create_role(RoleName=self._role_dictionary["Name"],
                                         AssumeRolePolicyDocument=assume_role_doc,
                                         Description=self._role_dictionary["Description"])
        else:
            r = self._client.create_role(RoleName=self._role_dictionary["Name"],
                                         AssumeRolePolicyDocument=assume_role_doc)

        # process tags if they exist
        if "Tags" in self._role_dictionary.keys():
            tag_dict_list = []
            for tag_dict in self._role_dictionary["Tags"]:
                tag_dict_list.append(tag_dict)

            if len(tag_dict_list) > 0:
                _ = self._client.tag_role(RoleArn=r["Role"]["Arn"], Tags=tag_dict_list)

        return r


if __name__ == "__main__":

    iam_client = None

    try:
        parser = AWSAPIUtilsParser()
        args = parser.parse_args()

        role_dict = parser.load_config_dict()
        iam_client = APIRoleCreator(role_dict)
        response = iam_client.create_role()

        role_name = response["Role"]["RoleName"]

        print("\"{}\" created in IAM with ARN {}.".format(role_name, str(response["Role"]["Arn"])))

        for arn in role_dict["Policies"]:
            iam_client.attach_role_policy(arn)
            print("Policy with ARN \"{}\" attached to {} role.".format(arn, role_name))

        exit(0)

    except Exception as e:
        print(e)
        exit(-1)
    finally:
        if iam_client is not None:
            iam_client.close()
