# Copyright 2023-2024 by Advanced Technology Systems, LLC.
#
# Unless required by applicable law or agreed to in writing, this software is provided on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND.
#
# Licensed by Advanced Technology Systems, LLC.  The full license is available at:
# http://advtechnologysystems.com/license

import boto3

from ats.utils.api.aws_api_utils_parser import AWSAPIUtilsParser


class APILambdaFunctionCreator(object):

    def __init__(self, lambda_dictionary):
        self._lambda_dictionary = lambda_dictionary
        self._client = boto3.client("lambda")

    def close(self):

        self._client.close()

    def create_basic_function_from_s3(self):

        # have default values if not in JSON configuration
        if "MemoryMB" in self._lambda_dictionary.keys():
            megs_memory = self._lambda_dictionary["MemoryMB"]
        else:
            megs_memory = 128
        if "PythonRuntime" in self._lambda_dictionary.keys():
            runtime_str = self._lambda_dictionary["PythonRuntime"]
        else:
            runtime_str = "python3.10"
        if "TimeoutSeconds" in self._lambda_dictionary.keys():
            timeout_secs = self._lambda_dictionary["TimeoutSeconds"]
        else:
            timeout_secs = 899
        if "DiskMB" in self._lambda_dictionary.keys():
            megs_disk = self._lambda_dictionary["DiskMB"]
        else:
            megs_disk = 512
        if "EntryPoint" in self._lambda_dictionary.keys():
            entry_point = self._lambda_dictionary["EntryPoint"]
        else:
            entry_point = "main.handle_event"

        code_dict = {"S3Bucket": self._lambda_dictionary["DeploymentBucket"],
                     "S3Key": self._lambda_dictionary["DeploymentKey"]}
        storage_dict = {"Size": megs_disk}

        if "Description" in self._lambda_dictionary.keys():
            r = self._client.create_function(FunctionName=self._lambda_dictionary["FunctionName"], Runtime=runtime_str,
                                             Role=self._lambda_dictionary["ExecutionRole"],
                                             Handler=entry_point, Code=code_dict, Timeout=timeout_secs,
                                             Publish=True, PackageType="Zip", MemorySize=megs_memory,
                                             Environment=self._lambda_dictionary["Environment"],
                                             EphemeralStorage=storage_dict,
                                             Description=self._lambda_dictionary["Description"])
        else:
            r = self._client.create_function(FunctionName=self._lambda_dictionary["FunctionName"], Runtime=runtime_str,
                                             Role=self._lambda_dictionary["ExecutionRole"],
                                             Handler=entry_point, Code=code_dict, Timeout=timeout_secs,
                                             Publish=True, PackageType="Zip", MemorySize=megs_memory,
                                             Environment=self._lambda_dictionary["Environment"],
                                             EphemeralStorage=storage_dict)

        # add permissions if they exist
        if "Permissions" in self._lambda_dictionary.keys():
            for permission_dict in self._lambda_dictionary["Permissions"]:
                _ = self._client.add_permission(FunctionName=self._lambda_dictionary["FunctionName"],
                                                StatementId=permission_dict["StatementId"],
                                                Action=permission_dict["Action"],
                                                Principal=permission_dict["Principal"],
                                                SourceArn=permission_dict["SourceArn"])

        # add tags if they exist
        if "Tags" in self._lambda_dictionary.keys():
            tags_dict = self._lambda_dictionary["Tags"]

            _ = self._client.tag_resource(Resource=r["FunctionArn"], Tags=tags_dict)

        return r


if __name__ == "__main__":

    lambda_client = None

    try:
        parser = AWSAPIUtilsParser()
        args = parser.parse_args()

        lambda_dict = parser.load_config_dict()
        lambda_client = APILambdaFunctionCreator(lambda_dict)
        response = lambda_client.create_basic_function_from_s3()

        print("Lambda function \"{}\" created with ARN {}.".format(response["FunctionName"], response["FunctionArn"]))

        exit(0)

    except Exception as e:
        print(e)
        exit(-1)
    finally:
        if lambda_client is not None:
            lambda_client.close()
